@@ -201,7 +201,8 @@ def ToList(input):
 
 _USED_DIRS = set()
 
-def _PyScript(env, target, source, command = "python $SOURCES", capture=None, execute=None, tempdir=None, lightweight=True, *args, **kwargs): 
+def _PyScript(env, target, source, command = "python $SOURCES", capture=None,
+        execute=None, tempdir=None, exit=False, lightweight=True, *args, **kwargs): 
     """
     A new Builder added to a scons environment which has the same syntax as Command.
     """
@@ -228,10 +229,10 @@ def _PyScript(env, target, source, command = "python $SOURCES", capture=None, ex
     command_sig = [ env.subst(c, target=target, source=subst) for c in command ] 
 
     basename = None
-    if execute and tempdir == None:
+    if (execute or exit) and tempdir == None:
         m = hashlib.sha1()
         m.update("\n".join(command_sig))
-        basename = os.path.join("temp", base64.b64encode(m.digest(), "+="), '')
+        basename = os.path.join("temp", "." + base64.b64encode(m.digest(), "+="), '')
     else: basename = tempdir
 
     if env.GetOption("clean") and basename:
@@ -240,7 +241,20 @@ def _PyScript(env, target, source, command = "python $SOURCES", capture=None, ex
         if not os.path.exists(basename): mkdir_p(basename)
         scons = os.path.join(basename, "SConscript")
         file = open(scons, 'w')
-        print >>file, "from pyscons import *\ntargets, sources, basename = PySconsSetup()\nPyScript(targets, sources, %s)" % repr(command)
+        print >>file, """from pyscons import *
+targets, sources, basename = PySconsSetup()
+
+env = DefaultEnvironment()
+
+command_sig = %s
+
+stargets = [t + ".shadow" for t in targets]
+command =  [env.subst(c, target=stargets, source=sources) for c in %s ]
+command += ["mv %%s %%s" %% (s, t) for s,t in zip(stargets, targets)]
+
+gen = lambda source, target, env, for_signature: command_sig if for_signature else command
+PyScript(targets, sources, CommandGeneratorAction(gen,{}))
+""" % (repr(command_sig), repr(command))
         file.close()
 
         arg = " ".join(["in%g=%s" % (n+1, str(s)) for n,s in enumerate(source)])
@@ -266,6 +280,7 @@ def _PyScript(env, target, source, command = "python $SOURCES", capture=None, ex
         if not lightweight: command = exe + " -Q"
 
     if execute: command = execute + " " + basename
+    if exit and "exit" in ARGUMENTS: command = "exit %s # %s" % (ARGUMENTS["exit"], basename)
 
     ftargets = [t + ".force" for t in target]
     if reduce(lambda a,b: a&b, (os.path.exists(t) for t in ftargets)):
@@ -274,10 +289,11 @@ def _PyScript(env, target, source, command = "python $SOURCES", capture=None, ex
     gen = lambda source, target, env, for_signature: command_sig if for_signature else command
     return env.Command(target, source, CommandGeneratorAction(gen,{}), *args, **kwargs)
 
-def _PyScons(env, target, source, scons, execute=None, options="-Q", sig_options="", clean = True, tempdir = None):
+def _PyScons(env, target, source, scons, execute=None, options="-Q",
+        sig_options="", clean = True, tempdir = None, exit=False):
     global _USED_DIRS
-    target = ToList(target) 
-    source = ToList(source) 
+    target = ToList(target)
+    source = ToList(source)
 
     args = " ".join(["in%g=%s" % (n+1, str(s)) for n,s in enumerate(source)])
     args += " " + " ".join(["out%g=%s" % (n+1, str(s)) for n,s in enumerate(target)])
@@ -286,7 +302,7 @@ def _PyScons(env, target, source, scons, execute=None, options="-Q", sig_options
     if tempdir == None:
         m = hashlib.sha1()
         m.update(sig)
-        basename = os.path.join("temp", base64.b64encode(m.digest(), "+="), '')
+        basename = os.path.join("temp", "." + base64.b64encode(m.digest(), "+="), '')
     else: basename = tempdir
 
     USE_DIR = tuple(os.path.split(basename))
@@ -303,16 +319,17 @@ def _PyScons(env, target, source, scons, execute=None, options="-Q", sig_options
     file.close()
 
     ts = os.path.join(basename, "TARGETS")
-    file = open(ts, 'w')   
+    file = open(ts, 'w')
     print >>file, "\n".join(target)
-    file.close()   
+    file.close()
 
     ss = os.path.join(basename, "SOURCES")
-    file = open(ss, 'w')   
+    file = open(ss, 'w')
     print >>file, "\n".join(source + [scons, ts, ss, exe])
     file.close()
 
-    cmd = exe + " " + options
+    if "exit" in ARGUMENTS: options += " -k exit=%s" % ARGUMENTS["exit"]
+    cmd = " ".join([exe, options])
     cleancmd = cmd + " --clean"
 
     if env.GetOption("clean") and not clean: 
@@ -322,6 +339,8 @@ def _PyScons(env, target, source, scons, execute=None, options="-Q", sig_options
     if not execute: cmd = [cmd, cleancmd] if clean else [cmd]
     else: cmd = [execute + " " + basename]
 
+    if exit and "exit" in ARGUMENTS: command = "exit %s # %s" % (ARGUMENTS["exit"], basename)
+
     ftargets = [t + ".force" for t in target]
     if reduce(lambda a,b: a&b, (os.path.exists(t) for t in ftargets)):
         cmd = ["mv %s %s" %(f,t) for f,t in zip(ftargets,target)]
@@ -390,11 +409,8 @@ def PySconsSetup(env=None, args=ARGUMENTS, tools=['default', PYTOOL()]):
     ship_source = [s for s in sources if s not in exclude] + [exe, myname]
 
     if "print" not in args and os.path.exists(force):
-        import subprocess
-        print "tar xjf %s" % force
-        subprocess.call(["tar xjf ", force])
-        print "Move(%s,%s)" % (force, out)
-        shutil.mv(force, out)
+        env.Execute("tar xjf " + force)
+        env.Execute("mv %s %s" % (force, out))
         env.Exit(0)
 
     env.Command(ship, ship_source, "tar cjf $TARGET $SOURCES")
